/* cmd_run.c
 * Copyright (C) 2008 Gauvain Pocentek <gpocentek@linutop.com>
 * This file is part of lbackup
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 2 of the license.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#define _GNU_SOURCE

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#ifdef HAVE_STRING_H
#include <string.h>
#endif

#include <glib.h>
#include <glib/gprintf.h>
#include <gtk/gtk.h>

#include <libxfcegui4/libxfcegui4.h>

#include "lbackup.h"

gboolean done = FALSE;
int prg = 1;
int prg_max = 10;
int cur_step = 0;

typedef struct _dgui {
    GtkWidget *w;
    GtkWidget *pbar;
    GtkWidget *label;

    int p_nb;
} dgui;

gboolean
update_pbar (gpointer data)
{
    if (done)
        return FALSE;

    dgui *dg = (dgui *) data;
    gtk_progress_bar_pulse (GTK_PROGRESS_BAR (dg->pbar));
    return TRUE;
}

static void
end_cmd (GIOChannel *channel, dgui *dg, gboolean success)
{
    if (success)
        xfce_info (_("Your system has been correctly duplicated."));
    else
        xfce_warn (_
              ("An error occured during the duplication. Please try again."));
    g_io_channel_unref (channel);
    gtk_widget_destroy (dg->w);
}

static gboolean
update_label (GIOChannel *channel, GIOCondition condition, gpointer data)
{
    char buf[BUFSIZ];
    guint fd;
    FILE *fp;
    dgui *dg = (dgui *) data;

    if (condition == G_IO_IN) {
        fd = g_io_channel_unix_get_fd (channel);
        fp = fdopen (fd, "r");
        fgets (buf, BUFSIZ, fp);
        if (strncmp (buf, "Success.", 8) == 0) {
            done = TRUE;
            end_cmd (channel, dg, TRUE);
            return FALSE;
        } else if (strncmp (buf, "Failure.", 8) == 0) {
            done = TRUE;
            end_cmd (channel, dg, FALSE);
            return FALSE;
        } else {
            gtk_label_set_text (GTK_LABEL (dg->label), buf);
            return TRUE;
        }
    } else {
        done = TRUE;
        xfce_warn (_("A problem occured. Please try again."));
        gtk_widget_destroy (dg->w);
        g_io_channel_unref (channel);
        return FALSE;
    }
}

void run_dialog
(GtkWidget *parent, gchar *device, gui *mgui)
{
    dgui *dg;        /* display gui */
    GtkWidget *vbox, *label;

    GIOChannel *channel;
    gint in, out, err;
    gchar *sargv[5];
    gchar *binary;

    dg = g_new0 (dgui, 1);
    dg->p_nb = 1;

    dg->w = gtk_dialog_new_with_buttons (_("Copying the system"),
                        GTK_WINDOW (parent),
                        GTK_DIALOG_MODAL |
                        GTK_DIALOG_DESTROY_WITH_PARENT |
                        GTK_DIALOG_NO_SEPARATOR, NULL);
    gtk_window_set_deletable (GTK_WINDOW (dg->w), FALSE);
    gtk_container_set_border_width (GTK_CONTAINER (dg->w), 6);
    vbox = gtk_vbox_new (FALSE, 5);
    gtk_container_add (GTK_CONTAINER (GTK_DIALOG (dg->w)->vbox), vbox);
    gtk_widget_show (vbox);

    label = gtk_label_new (NULL);
    gtk_label_set_markup (GTK_LABEL (label),
                 _("<span weight=\"bold\" style=\"italic\">"
                   "Please wait during the copy (this will take several minutes)</span>"));
    gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, FALSE, 5);
    gtk_widget_show (label);

    dg->label = gtk_label_new (NULL);
    gtk_box_pack_start (GTK_BOX (vbox), dg->label, FALSE, FALSE, 5);
    gtk_widget_show (dg->label);

    dg->pbar = gtk_progress_bar_new ();
    gtk_progress_bar_set_pulse_step (GTK_PROGRESS_BAR (dg->pbar), 0.1);
    gtk_box_pack_start (GTK_BOX (vbox), dg->pbar, FALSE, FALSE, 5);
    gtk_widget_show (dg->pbar);

    /* update the label with some '.' every second */
    g_timeout_add (100, (GSourceFunc) update_pbar, dg);

    binary = g_strdup (mgui->backup ? "/usr/bin/lusm-backup" : "/usr/bin/lusm");

    sargv[0] = g_strndup (binary, strlen (binary));
    sargv[1] = g_strdup ("-y");
    sargv[2] = g_strdup ("-d");
    sargv[3] = g_strndup (device, strlen (device));
    sargv[4] = NULL;

    g_spawn_async_with_pipes (NULL, sargv, NULL,
                 G_SPAWN_LEAVE_DESCRIPTORS_OPEN, NULL, NULL,
                 NULL, &in, &out, &err, NULL);
    channel = g_io_channel_unix_new (out);
    g_io_add_watch (channel, G_IO_IN, (GIOFunc) update_label, dg);

    gtk_dialog_run (GTK_DIALOG (dg->w));

    int i;
    for (i=0;i<4;i++)
        g_free (sargv[i]);
    g_free (dg);
    g_free (binary);
}

void
run_lusm (GtkWidget *parent, gui *mgui)
{
    gboolean ok;
    gchar warn_msg[95];
    gchar *device = NULL;
    GtkTreeIter iter;
    GtkTreeModel *model;

    model = gtk_combo_box_get_model (GTK_COMBO_BOX (mgui->combo));
    gtk_combo_box_get_active_iter (GTK_COMBO_BOX (mgui->combo), &iter);
    gtk_tree_model_get (model, &iter, 0, &device, -1);

    g_sprintf (warn_msg,
          _
          ("The content of %s will be erased.\nDo you really want to continue?"),
          device);
    ok = xfce_confirm (warn_msg, GTK_STOCK_OK, NULL);
    if (!ok)
        return;

    run_dialog (parent, device, mgui);
    g_free (device);
}
