/**
 * Copyright (C) 2008 Gauvain Pocentek <gpocentek@linutop.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "main.h"
#include "syscfg.h"

#define NEXTTIMEDESKTOPFILE DATADIR "/applications/lconfeditor.desktop"
#define ACPICONFFILE "/etc/default/eeepc-acpi-scripts"

enum init_ids {
    INIT_SCREEN = 0,
    INIT_VIDEOOUTPUT,
    INIT_PERSISTENT,
    INIT_LOCK,
    INIT_VNC,
    INIT_NM,
};

/*
 * generic functions
 */
inline static void
set (GObject *obj, gpointer value)
{
    g_object_set_data (obj, "ltopconf", value);
}

inline static gpointer
get (GObject *obj)
{
    return g_object_get_data (obj, "ltopconf");
}

/*
 * per setting functions
 */
/* language */
void
set_locale (const gchar *code)
{
    set (G_OBJECT (dialog->locale_btn), (gpointer) g_strdup (code));
    gtk_button_set_label (GTK_BUTTON (dialog->locale_btn),
        get_locale_name (code));
}

const gchar *
get_locale ()
{
    const gchar *tmp =  (const gchar *) get (G_OBJECT (dialog->locale_btn));
    return g_strconcat (" locale=", tmp, NULL);
}

const gchar *
get_locale_value ()
{
    return  (const gchar *) get (G_OBJECT (dialog->locale_btn));
}

/* kbd */
inline void
set_kbd_user_defined (gboolean b)
{
    g_object_set_data (G_OBJECT (dialog->kbd_btn), "user_defined",
        (gpointer) b);
}

inline gboolean
get_kbd_user_defined ()
{
    return (gboolean) g_object_get_data (G_OBJECT (dialog->kbd_btn), "user_defined");
}

void
set_kbd (const gchar *code)
{
    set (G_OBJECT (dialog->kbd_btn), (gpointer) g_strdup (code));
    gtk_button_set_label (GTK_BUTTON (dialog->kbd_btn),
        (const char *) (get_kbd_name (code)));
}

const gchar *
get_kbd ()
{
    const gchar *code = (const gchar *) get (G_OBJECT (dialog->kbd_btn));
    gchar *tmp = g_strconcat (" console-setup/layoutcode=", code,
                              " xkbdlayout=", code, NULL);
    const gchar *ret = g_strdup (tmp);
    g_free (tmp);
    return ret;
}

inline const gchar *
get_kbd_value ()
{
    return (const gchar *) get (G_OBJECT (dialog->kbd_btn));
}

inline static const gchar *
get_init_kbd_value ()
{
    return (const gchar *) g_object_get_data (G_OBJECT (dialog->dialog), "init_kbd");
}

void
restore_kbd_config ()
{
    const gchar *tmp = get_init_kbd_value ();
    apply_kbd_config (tmp);
}

/* timezone */
void
inline set_tz_user_defined (gboolean b)
{
    g_object_set_data (G_OBJECT (dialog->tz_btn), "user_defined",
        (gpointer) b);
}

gboolean
inline get_tz_user_defined ()
{
    return (gboolean) g_object_get_data (G_OBJECT (dialog->tz_btn), "user_defined");
}

void
set_tz (const gchar *code)
{
    set (G_OBJECT (dialog->tz_btn), (gpointer) g_strdup (code));
    gtk_button_set_label (GTK_BUTTON (dialog->tz_btn),
        code);
}

const gchar *
get_tz ()
{
    const gchar *tmp = (const gchar *) get (G_OBJECT (dialog->tz_btn));
    return g_strconcat (" timezone=", tmp, NULL);
}

inline const gchar *
get_tz_value ()
{
    return (const gchar *) get (G_OBJECT (dialog->tz_btn));
}

/* screen */
void
set_screensize (const gchar *code)
{
    if (code == NULL)
    {
        gtk_combo_box_set_active (GTK_COMBO_BOX (dialog->screen_combo), 0);
        return;
    }

    int i = 1;
    gboolean found_it = FALSE;
    while (resolutions[i].code != NULL) {
        if (!strcmp (code, resolutions[i].code))
        {
            gtk_combo_box_set_active (
                GTK_COMBO_BOX (dialog->screen_combo), i);
            found_it = TRUE;
            break;
        }
        if (!strcmp ("custom", resolutions[i].code) && IS_L3)
        {
            gtk_combo_box_set_active (
                GTK_COMBO_BOX (dialog->screen_combo), i);
            found_it = TRUE;
            break;
        }
        i++;
    }
    if (!found_it)
        gtk_combo_box_set_active (GTK_COMBO_BOX (dialog->screen_combo), 0);

    if (resolutions[i].code && !strcmp(resolutions[i].code, "custom"))
    {
        gchar **split = g_strsplit (code, "x", 0);
        if (g_strv_length (split) != 2)
        {
            /* something went wrong, restore a valid conf */
            gtk_combo_box_set_active (GTK_COMBO_BOX (dialog->screen_combo), 0);
            g_strfreev (split);
            return;
        }

        g_object_set_data (G_OBJECT (dialog->screen_combo),
            "width", (gpointer) g_strdup (split[0]));
        g_object_set_data (G_OBJECT (dialog->screen_combo),
            "height", (gpointer) g_strdup (split[1]));

        g_strfreev (split);
    }
}

const gchar *
get_screen ()
{
    const gchar *ret;
    const gchar *value = get_screen_value ();
    if (!strcmp (value, ""))
        return "";
    ret = (const gchar *) g_strconcat (" screensize=", value, NULL);
    return ret;
}

const gchar *
get_screen_value ()
{
    GtkTreeModel *model;
    GtkTreeIter iter;
    const gchar *ret;
    gchar *tmp = NULL;

    model = gtk_combo_box_get_model (GTK_COMBO_BOX (dialog->screen_combo));
    gtk_combo_box_get_active_iter (GTK_COMBO_BOX (dialog->screen_combo), &iter);
    gtk_tree_model_get (model, &iter, COL_SCREEN_CODE, &tmp, -1);

    if (!strcmp ("auto", tmp))
    {
        g_free (tmp);
        return "";
    }
    if (!strcmp ("custom", tmp))
    {
        g_free (tmp);
        gchar *w = NULL, *h = NULL;
        w = (gchar *) g_object_get_data (
            G_OBJECT (dialog->screen_combo), "width");
        h = (gchar *) g_object_get_data (
            G_OBJECT (dialog->screen_combo), "height");

        if (!w || !h)
        {
            return "";
        }
        else
        {
            ret = g_strdup_printf ("%sx%s", w, h);
            return ret;
        }
    }

    ret = g_strdup (tmp);
    g_free (tmp);

    return ret;
}

void
set_output (guint code)
{
    int i = 0;
    gboolean found_it = FALSE;
    while (videooutputs[i].code != 0) {
        if (code == videooutputs[i].code)
        {
            gtk_combo_box_set_active (
                GTK_COMBO_BOX (dialog->videooutput_combo), i);
            found_it = TRUE;
            break;
        }
        i++;
    }
    if (!found_it)
        gtk_combo_box_set_active (GTK_COMBO_BOX (dialog->videooutput_combo), 0);
}

const gchar *
get_output ()
{
    if (!IS_L3)
        return "";

    const gchar *ret;
    guint value = get_output_value ();
    if (!value)
        return "";
    ret = g_strdup_printf (" videooutput=%d", value);
    return ret;
}

guint
get_output_value ()
{
    GtkTreeModel *model;
    GtkTreeIter iter;
    guint ret;

    model = gtk_combo_box_get_model (GTK_COMBO_BOX (dialog->videooutput_combo));
    gtk_combo_box_get_active_iter (GTK_COMBO_BOX (dialog->videooutput_combo), &iter);
    gtk_tree_model_get (model, &iter, COL_OUTPUT_CODE, &ret, -1);

    return ret;
}

/* show next time */
inline void
set_next_time (gboolean nt)
{
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dialog->next_time), nt);
}

inline gboolean
get_next_time ()
{
    return gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dialog->next_time));
}

/* no X */
inline void
set_no_x (gboolean nox)
{
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dialog->nox_chk), nox);
}

inline const gchar *
get_no_x ()
{
    return gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dialog->nox_chk))
        ? " noX"
        : "";
}

/* checkfs */
inline void
set_checkfs (gboolean check)
{
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dialog->checkfs_chk), check);
}

inline const gchar *
get_checkfs ()
{
    return gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dialog->checkfs_chk))
        ? " checkfs"
        : "";
}

/* reboot schedule */
void
set_reboot_sched (gboolean sched, guint hour, guint minute)
{
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dialog->reboot_chk), sched);
    gtk_spin_button_set_value (GTK_SPIN_BUTTON (dialog->reboot_min_spin),
                               (gdouble) minute);
    gtk_spin_button_set_value (GTK_SPIN_BUTTON (dialog->reboot_hour_spin),
                               (gdouble) hour);
}

gboolean
get_reboot_sched (guint *hour, guint *minute)
{
    if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dialog->reboot_chk)))
    {
        *minute = (guint) gtk_spin_button_get_value (
                            GTK_SPIN_BUTTON (dialog->reboot_min_spin));
        *hour = (guint) gtk_spin_button_get_value (
                            GTK_SPIN_BUTTON (dialog->reboot_hour_spin));
        return TRUE;
    }
    else
    {
        *minute = 0;
        *hour = 0;
        return FALSE;
    }
}

/* persistence */
inline void
set_persistent (const gchar *persistent)
{
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (
         (strncmp (persistent, "full", 4) == 0) ? dialog->full_chk : dialog->home_chk),
         TRUE);
}

inline const gchar *
get_persistent ()
{
    return gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dialog->full_chk))
        ? " persistent=full"
        : "";
}

inline const gchar *
get_persistent_value ()
{
    return gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dialog->full_chk))
        ? "full"
        : "";
}

/* lock active*/
void
set_lock (gboolean locked)
{
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dialog->lock_chk), locked);
}

inline gboolean
get_lock ()
{
    return gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dialog->lock_chk));
}

/* lid */
void
set_lid_action ()
{
    if (! IS_LAPTOP)
        return;
    int id = LID_NONE;
    XfceRc *rc = xfce_rc_simple_open (ACPICONFFILE, TRUE);

    if (rc)
    {
        gchar *tmp = g_strdup (xfce_rc_read_entry (rc, "LID_ACTION", "none"));
        xfce_rc_close (rc);
        gchar *value = g_shell_unquote (tmp, NULL);
        g_free (tmp);
        if (strncmp (value, "suspend", 4) == 0)
            id = LID_SUSPEND;
        else if (strncmp (value, "halt", 4) == 0)
            id = LID_HALT;

        g_free (value);
    }
    gtk_combo_box_set_active (GTK_COMBO_BOX (dialog->lid_combo), id);
}

inline int
get_lid_action ()
{
    return gtk_combo_box_get_active (GTK_COMBO_BOX (dialog->lid_combo));
}

void
write_lid_action ()
{
    if (! IS_LAPTOP)
        return;

    const gchar *value;
    int id = get_lid_action ();
    switch (id)
    {
        case LID_HALT:
            value = "halt";
            break;
        case LID_SUSPEND:
            value = "suspend";
            break;
        case LID_NONE:
        default:
            value = "none";
            break;
    }

    XfceRc *rc = xfce_rc_simple_open (ACPICONFFILE, FALSE);
    if (!rc)
        return;
    gchar *tmp = g_shell_quote (value);
    xfce_rc_write_entry (rc, "LID_ACTION", tmp);
    xfce_rc_close (rc);
    g_free (tmp);
}

/* nm settings */
inline void
set_nm (gboolean nm)
{
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dialog->nm_chk), nm);
}

inline gboolean
get_nm ()
{
    return gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dialog->nm_chk));
}

/* reboot */
inline void
set_reboot_needed (gboolean needed)
{
    g_object_set_data (G_OBJECT (dialog->dialog), "reboot",
        (gpointer) needed);
}

inline gboolean
get_reboot_needed ()
{
    return (gboolean) g_object_get_data (G_OBJECT (dialog->dialog), "reboot");
}

/* update kbd and tz if needed */
void
update_locale_settings (const gchar *locale)
{
    gboolean kbd_user = get_kbd_user_defined ();
    gboolean tz_user = get_tz_user_defined ();
    if (kbd_user && tz_user)
        return;

    gchar *kbd, *tz;
    get_from_language (locale, &kbd, &tz);

    if (!kbd_user)
    {
        set_kbd ((const gchar *)kbd);
        apply_kbd_config ((const gchar *)kbd);
    }
    if (!tz_user)
        set_tz ((const gchar *)tz);

    g_free (kbd);
    g_free (tz);
}


static gpointer
get_initial_conf (int id)
{
    switch (id)
    {
        case INIT_SCREEN:
            return g_object_get_data (G_OBJECT (dialog->dialog),
                "init_screen");
        case INIT_VIDEOOUTPUT:
            return g_object_get_data (G_OBJECT (dialog->dialog),
                "init_videooutput");
        case INIT_PERSISTENT:
            return g_object_get_data (G_OBJECT (dialog->dialog),
                "init_persistent");
        case INIT_LOCK:
            return g_object_get_data (G_OBJECT (dialog->dialog),
                "init_lock");
        case INIT_NM:
            return g_object_get_data (G_OBJECT (dialog->dialog),
                "init_nm");
    }
    return (gpointer) NULL;
}

static void
set_initial_conf (gchar *screen, gchar *persistent,
                  gchar *kbd, gboolean lock,
                  gboolean nm, guint video)
{
    g_object_set_data (G_OBJECT (dialog->dialog), "init_screen",
        (gpointer) g_strdup (screen));
    g_object_set_data (G_OBJECT (dialog->dialog), "init_videooutput",
        GINT_TO_POINTER (video));
    g_object_set_data (G_OBJECT (dialog->dialog), "init_persistent",
        (gpointer) g_strdup (persistent));
    g_object_set_data (G_OBJECT (dialog->dialog), "init_kbd",
        (gpointer) g_strdup (kbd));
    g_object_set_data (G_OBJECT (dialog->dialog), "init_lock",
        (gpointer) lock);
    g_object_set_data (G_OBJECT (dialog->dialog), "init_nm",
        (gpointer) nm);
}

static gboolean
need_reboot ()
{
    const gchar *init, *now;
    gboolean linit, lnow;

    /* screen */
    init = (const gchar *) get_initial_conf (INIT_SCREEN);
    now = get_screen_value ();
    if (strcmp (init, now) != 0)
        return TRUE;

    /* video output */
    if (IS_L3)
    {
        guint iinit, inow;
        iinit = (guint) get_initial_conf (INIT_VIDEOOUTPUT);
        inow = get_output_value ();
        if (iinit != inow)
            return TRUE;
    }

    /* persistence */
    init = (const gchar *) get_initial_conf (INIT_PERSISTENT);
    now = get_persistent_value ();
    if (strcmp (init, now) != 0)
        return TRUE;

    /* lock */
    linit = (gboolean) get_initial_conf (INIT_LOCK);
    lnow = get_lock ();
    if (linit != lnow)
        return TRUE;

    /* NM */
    if (! IS_LAPTOP)
    {
        linit = (gboolean) get_initial_conf (INIT_NM);
        lnow = get_nm ();
        if (linit != lnow)
        {
            /* apply the changes here */
            char *command =
                g_strconcat ("sh ", LIBEXECDIR, "/enable-nm.sh ",
                             lnow ? "true" : "false",
                             NULL);
            system (command);
            g_free (command);

            return TRUE;
        }
    }

    return FALSE;
}

/* apply settings */
static void
change_next_time()
{
    gchar dir[64];
    gchar cmd[128];
    gchar *tmp = NULL;
    gchar *home = NULL;
    uid_t uid = 1000;

    tmp = getenv("SUDO_UID");
    if (tmp)
        uid = atoi(tmp);

    home = get_home ();
    g_sprintf(dir, "%s/.config/autostart/", home);
    g_free (home);
    mkdir(dir, 0755);

    if (get_next_time ()) {
        g_sprintf(cmd, "cp %s %s", NEXTTIMEDESKTOPFILE, dir);
        system((const char *)cmd);
        g_sprintf(cmd, "%s/lconfeditor.desktop", dir);
        chown(dir, uid, uid);
    } else {
        g_sprintf(cmd, "%s/lconfeditor.desktop", dir);
        unlink(cmd);
    }
}

static void
schedule_reboot ()
{
    gchar *cmd = NULL;
    guint reboot_min;
    guint reboot_hour;

    if (get_reboot_sched (&reboot_hour, &reboot_min))
        cmd = g_strdup_printf ("sh " LIBEXECDIR "/reboot.sh %d %d",
                               reboot_hour, reboot_min);
    else
        cmd = g_strdup_printf ("sh " LIBEXECDIR "/reboot.sh cancel");

    system (cmd);
    g_free (cmd);
}

/**********************************/
void
read_config ()
{
    FILE *fp;
    gchar nexttime_txt[64];

    gchar *tz_txt = g_strdup ("Europe/London");
    gchar *locale_txt = g_strdup ("en_US.UTF-8");
    gchar *kbd_code_txt = g_strdup ("us");
    gchar *screen_txt = g_strdup ("");
    gchar *persistent = g_strdup ("");
    guint video_code = 1;
    guint reboot_min = 0;
    guint reboot_hour = 0;
    gboolean nox = FALSE;
    gboolean checkfs = FALSE;
    gboolean nexttime = FALSE;
    gboolean lock = FALSE;
    gboolean nm;

    dialog->base_cmdline = g_strdup ("append ");

    dialog->lock_pwd = NULL;

    if (G_LIKELY((fp = fopen(CONFFILE, "r")))) {
        gchar *gline = NULL, *lline = NULL;
        gchar *tmp;
        size_t len = 0;
        ssize_t read;

        while ((read = getline(&tmp, &len, fp)) != -1) {
            if (!gline && (strncmp(tmp, "append  ", 8) == 0)) {
                g_free (gline);
                gline = g_strdup(tmp);
                if (gline[strlen(gline) - 1] == '\n')
                    gline[strlen(gline) - 1] = '\0';
            }
            if (!lline
                && (strncmp(tmp, "MENU MASTER PASSWD", 18) == 0)) {
                    lock = TRUE;
                    lline = g_strdup(tmp);
                    if (lline[strlen(lline) - 1] == '\n')
                        lline[strlen(lline) - 1] = '\0';
                    gchar **exploded = g_strsplit(lline, " ", 0);
                    if (g_strv_length(exploded) == 4)
                        dialog->lock_pwd =
                            g_strdup(exploded[3]);
                    g_strfreev(exploded);
                }
        }
        g_free(lline);
        g_free(tmp);
        fclose(fp);

        if (G_LIKELY(gline)) {
            gchar **exploded = g_strsplit(gline, " ", 0);
            int count = 0;
            while (exploded[count] != '\0') {
                if (*exploded[count] == '\0')
                {
                    count++;
                    continue;
                }
                gchar **item =
                    g_strsplit(exploded[count], "=", 0);
                if (g_strv_length(item) == 2) {
                    if (item[0] && item[1]
                        && strncmp(item[0], "xkbdlayout", 10) == 0)
                    {
                        g_free (kbd_code_txt);
                        kbd_code_txt = g_strdup (item[1]);
                    }
                    else if (item[0] && item[1]
                        && strncmp(item[0], "locale", 6) == 0)
                    {
                        g_free (locale_txt);
                        locale_txt = g_strdup (item[1]);
                    }
                    else if (item[0] && item[1]
                        && strncmp(item[0], "timezone", 8) == 0)
                    {
                        g_free (tz_txt);
                        tz_txt =g_strdup (item[1]);
                    }
                    else if (item[0] && item[1]
                        && strncmp(item[0], "screensize", 10) == 0)
                    {
                        g_free (screen_txt);
                        screen_txt = g_strdup (item[1]);
                    }
                    else if (item[0] && item[1]
                        && strncmp(item[0], "videooutput", 11) == 0)
                    {
                        video_code = atoi (item[1]);
                        if (video_code < 0 || video_code == 2 || video_code > VIDEO_INVALID)
                            video_code = 1;
                    }
                    else if (item[0] && item[1]
                        && strncmp(item[0], "persistent", 10) == 0)
                    {
                        g_free (persistent);
                        persistent = g_strdup (item[1]);
                    }
                    else if (item[0] &&
                        strncmp(item[0], "console-setup/layoutcode", 24) == 0)
                    {
                        /* do nothing, but don't register it as an unknown parameter */
                    }
                    /* we don't handle the item, store it */
                    else
                    {
                        gchar *tmp = g_strconcat (dialog->base_cmdline, " ", exploded[count], NULL);
                        g_free (dialog->base_cmdline);
                        dialog->base_cmdline = tmp;
                    }

                } else {
                    if (strncmp (exploded[count], "noX", 3) == 0)
                        nox = TRUE;
                    else if (strncmp (exploded[count], "checkfs", 7) == 0)
                        checkfs = TRUE;
                    else if ((strncmp (exploded[count], "append", 6) == 0)
                             || (strncmp (exploded[count], "lock", 4) == 0))
                    {
                        /* do nothing, but don't register it as an unknown parameter */
                    }
                    else
                    {
                        gchar *tmp = g_strconcat (dialog->base_cmdline, " ", exploded[count], NULL);
                        g_free (dialog->base_cmdline);
                        dialog->base_cmdline = tmp;
                    }
                }
                g_strfreev(item);
                count++;
            }
            g_free(gline);
            g_strfreev(exploded);
        }
    }

    g_sprintf(nexttime_txt, "%s/.config/autostart/lconfeditor.desktop",
          get_home());
    if ((g_access("/cdrom/lconfeditor-stamp", R_OK) != 0)
        || (g_access(nexttime_txt, R_OK) == 0)) {
            nexttime = TRUE;
    }

    nm = !get_service_is_blacklisted ("NetworkManager");

    if (reboot_scheduled (&reboot_hour, &reboot_min))
        set_reboot_sched (TRUE, reboot_hour, reboot_min);
    else
        set_reboot_sched (FALSE, reboot_hour, reboot_min);
    set_lid_action ();
    set_nm (nm);
    set_locale (locale_txt);
    set_kbd (kbd_code_txt);
    set_tz (tz_txt);
    set_screensize (screen_txt);
    set_next_time (nexttime);
    set_no_x (nox);
    set_checkfs (checkfs);
    set_persistent (persistent);
    set_lock (lock);
    set_initial_conf (screen_txt, persistent,
                      kbd_code_txt, lock,
                      nm, video_code);
    set_reboot_needed (FALSE);
    set_kbd_user_defined (FALSE);
    set_tz_user_defined (FALSE);

    if (IS_L3)
        set_output (video_code);

    g_free (tz_txt);
    g_free (locale_txt);
    g_free (kbd_code_txt);
    g_free (screen_txt);
    g_free (persistent);
}

void
write_config ()
{
    gchar *tz_txt = g_strdup (get_tz ());
    gchar *locale_txt = g_strdup (get_locale ());
    gchar *kbd_code_txt = g_strdup (get_kbd ());
    gchar *screen_txt = g_strdup (get_screen ());
    gchar *videooutput_txt = g_strdup (get_output ());
    gchar *persistent = g_strdup (get_persistent ());
    gchar *nox = g_strdup (get_no_x ());
    gchar *checkfs = g_strdup (get_checkfs ());
    gboolean lock = get_lock ();
    gchar *str = NULL;
    gchar output[BUFSIZ];
    FILE *f;

    str = g_strconcat (
        dialog->base_cmdline,
        tz_txt,
        locale_txt,
        kbd_code_txt,
        screen_txt,
        videooutput_txt,
        persistent,
        nox,
        checkfs,
        NULL);

    if (lock)
        g_sprintf (output, locked_syscfg, dialog->lock_pwd, str);
    else
        g_sprintf (output, default_syscfg, str);
    if ((f = fopen (CONFFILE, "w"))) {
        fputs (output, f);
        fclose (f);
        if (G_LIKELY ((f = fopen ("/cdrom/lconfeditor-stamp", "w"))))
            fclose (f);
    }

    /* blacklist gdm service if noX */
    if (!strcmp (nox, ""))
        system ("services-blacklist -n del gdm");
    else
        system ("services-blacklist -n add gdm");

    /* apply other settings */
    change_next_time ();
    write_lid_action ();
    schedule_reboot ();
    if (need_reboot ())
        set_reboot_needed (TRUE);

    g_free (tz_txt);
    g_free (locale_txt);
    g_free (kbd_code_txt);
    g_free (screen_txt);
    g_free (persistent);
    g_free (nox);
    g_free (checkfs);
    g_free (str);
}
