/* language.c
 * Copyright (C) 2007-2008 Gauvain Pocentek <gpocentek@linutop.com>
 * This file is part of the linutop config editor
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 2 of the license.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#ifdef HAVE_SYS_TYPES_H
#include <sys/types.h>
#endif

#include <gtk/gtk.h>

#include <libxfce4util/libxfce4util.h>

#include "utils.h"

/* keyboard globals */
gchar *l = NULL;
gchar *v = NULL;

static const char *dentry_keys [] = {
    "Name",
    "GenericName",
    "Exec",
    "Icon",
};

/* */
const struct resolution resolutions[] = {
    { "auto", N_("Automatic") },
    { "640x480", N_("640x480 (15'')") },
    { "800x600", N_("800x600 (15'')") },
    { "1024x768", N_("1024x768 (15'')") },
    { "1280x1024", N_("1280x1024 (17-19'')") },
    { "1280x768", N_("1280x768 (17'' wide)") },
    { "1366x768", N_("1366x768 (19'' wide)") },
    { "1600x1200", N_("1600x1200 (20'' wide)") },
    { "1680x1050", N_("1680x1050 (22'' wide)") },
    { "1920x1080", N_("1920x1080 (HD)") },
    { "1920x1440", N_("1920x1440 (Max)") },
    { "custom", N_("Other...") },
    { NULL, NULL }
};

const struct videooutput videooutputs[] = {
    { VIDEO_CRT, N_("VGA Only") },
    { VIDEO_DVI, N_("DVI Only") },
    { VIDEO_CRT_DVI, N_("Dual, VGA - DVI") },
    { VIDEO_DVI_CRT, N_("Dual, DVI - VGA") },
    { 0, NULL },
};

/* language data */
const struct locale_item locale_data[] = {
    { "Arabic", "ar_EG.UTF-8", "ara", "Africa/Cairo" },
    { "Czech", "cs_CZ.UTF-8", "cz", "Europe/Prague" },
    { "Danish", "da_DK.UTF-8", "dk", "Europe/Copenhagen" },
    { "Dutch", "nl_NL.UTF-8", "nl", "Europe/Amsterdam" },
    { "English", "en_US.UTF-8", "us", "Europe/London" },
    { "Finnish", "fi_FI.UTF-8", "fi", "Europe/Helsinki" },
    { "French", "fr_FR.UTF-8", "fr", "Europe/Paris" },
    { "German", "de_DE.UTF-8", "de", "Europe/Berlin" },
    { "Greek", "el_GR.UTF-8", "gr", "Europe/Athens" },
    { "Hungarian", "hu_HU.UTF-8", "hu", "Europe/Budapest" },
    { "Italian", "it_IT.UTF-8", "it", "Europe/Rome" },
    { "Japanese", "ja_JP.UTF-8", "jp", "Asia/Tokyo" },
    { "Norwegian Bokmaal", "nb_NO.UTF-8", "no", "Europe/Oslo" },
    { "Polish", "pl_PL.UTF-8", "pl", "Europe/Warsaw" },
    { "Portuguese", "pt_PT.UTF-8", "pt", "Europe/Lisbon" },
    { "Portuguese (Brazil)", "pt_BR.UTF-8", "pt", "Brazil/East" },
    { "Romanian", "ro_RO.UTF-8", "ro", "Europe/Bucharest" },
    { "Russian", "ru_RU.UTF-8", "ru", "Europe/Moscow" },
    { "Slovak", "sk_SK.UTF-8", "sk", "Europe/Bratislava" },
    { "Slovenian", "sl_SI.UTF-8", "si", "Europe/Ljubljana" },
    { "Spanish", "es_ES.UTF-8", "es", "Europe/Madrid" },
    { NULL, NULL, NULL, NULL }
};

/* GTK utils */
void
cb_row_activated (GtkWidget * dialog, gpointer data)
{
  gtk_dialog_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
}

GtkWidget *
get_image_from_str (const gchar *str)
{
    GtkWidget *image = NULL;

    if (g_str_has_prefix (str, "/"))
    {
        /* absolute path */
        GdkPixbuf *pxb = gdk_pixbuf_new_from_file_at_size (
            str, 18, 18, NULL);
        if (pxb)
        {
            image = gtk_image_new_from_pixbuf (pxb);
            g_object_unref (pxb);
        }
    }
    else
        image = gtk_image_new_from_icon_name (str, GTK_ICON_SIZE_SMALL_TOOLBAR);

    return image;
}

static void
run_btn_clicked (GtkWidget *btn, gpointer data)
{
    gchar *cmd = NULL;
    gboolean as_user;

    cmd = (gchar *) g_object_get_data (G_OBJECT (btn), "app");
    as_user = (gboolean) g_object_get_data (G_OBJECT (btn), "as_user");

    if (as_user)
        run_user_cmd (cmd, TRUE);
    else
        xfce_exec (cmd, FALSE, TRUE, NULL);
}

GtkWidget *
new_run_cmd_button (const gchar *icon, const gchar *txt,
                    const gchar *app, gboolean as_user)
{
    GtkWidget *btn, *image = NULL;
    gboolean usable;

    image = get_image_from_str (icon);

    btn = gtk_button_new ();
    if (image)
        gtk_button_set_image (GTK_BUTTON (btn), image);
    gtk_button_set_label (GTK_BUTTON (btn), txt);
    gtk_button_set_alignment (GTK_BUTTON (btn), 0, 0.5);
    g_object_set_data (G_OBJECT (btn), "app", (gpointer) app);
    g_object_set_data (G_OBJECT (btn), "as_user", (gpointer) as_user);
    usable = app_is_usable (app);
    g_object_set_data (G_OBJECT (btn), "usable", (gpointer) usable);
    gtk_widget_set_sensitive (btn, usable);

    g_signal_connect (G_OBJECT (btn), "clicked",
        G_CALLBACK (run_btn_clicked), NULL);

    return btn;
}

/* Text utils */
gboolean
app_is_usable (const gchar *app)
{
    gchar **spl = NULL;
    gboolean ret = FALSE;
    gchar *fullname = NULL;

    spl = g_strsplit (app, " ", 0);
    if (!spl)
        return FALSE;
    fullname = g_find_program_in_path (spl[0]);
    if (fullname && g_access (fullname, X_OK) == 0)
        ret = TRUE;

    g_strfreev (spl);
    g_free (fullname);

    return ret;
}

gchar*
get_home ()
{
    struct passwd *pw;
    uid_t uid = 0;
    gchar *tmp = NULL;
    gchar *ret = NULL;

    tmp = getenv ("SUDO_UID");
    if (tmp)
        uid = atoi (tmp);
    if (uid >= 0)
        pw = getpwuid (uid);
    if (pw)
        ret = g_strdup (pw->pw_dir);
    else
        ret = g_strdup ("root");

    return ret;
}

void
run_user_cmd (const char *user_cmd, gboolean sn)
{
    uid_t uid = 0;
    char *cuid = NULL;
    gchar cmd[100];
    gchar *run_sn = sn ? "" : "-n";

    cuid = getenv ("SUDO_UID");
    if (cuid)
        uid = atoi (cuid);
    if (uid >= 0) {
        g_sprintf (cmd, "lstart %s -u %d \"%s\"", run_sn, uid, user_cmd);
        xfce_exec (cmd, FALSE, FALSE, NULL);
    }
}

gchar *
get_locale_name (const gchar *code)
{
    gchar *name = NULL;
    int count = 0;
    while (locale_data[count].code != '\0') {
        if (!strncmp (locale_data[count].code, code, strlen (code))) {
            name = g_strdup (locale_data[count].name);
            return name;
        }
        count++;
    }
    return name;
}

/* kbd utils */
void
apply_kbd_config (const gchar *kbd)
{
    gchar cmd[100];
    g_sprintf (cmd, "setxkbmap %s", kbd);
    xfce_exec (cmd, FALSE, FALSE, NULL);
}

static void
get_variant_desc (XklConfigRegistry * config_registry,
                  XklConfigItem * config_item, gchar ** explode)
{
    if (strcmp (explode[1], config_item->name) == 0)
        v = g_strdup (config_item->description);
}

static void
get_layout_desc (XklConfigRegistry * config_registry,
        XklConfigItem * config_item, gchar ** explode)
{
    if (strcmp (explode[0], config_item->name) == 0) {
        l = g_strdup (config_item->description);
        if (explode[1]) {
            xkl_config_registry_foreach_layout_variant
                (config_registry, config_item->name,
                 (ConfigItemProcessFunc)
                 get_variant_desc, explode);
        }
    }
}

gchar *
get_kbd_name (const gchar *str)
{
    gchar **explode;
    gchar *ret;
    explode = g_strsplit (str, "_", 0);

    xkl_config_registry_foreach_layout (registry, (ConfigItemProcessFunc)
                       get_layout_desc, explode);

    if (explode[1])
        ret = g_strconcat (l, " (", v, ")", NULL);
    else
        ret = g_strdup (l);

    g_free (l);
    g_free (v);
    l = NULL;
    v = NULL;
    g_strfreev (explode);

    return ret;
}

void
get_from_language (const gchar *locale, gchar **kbd, gchar **tz)
{
    int i;
    gboolean found = FALSE;
    for (i=0; locale_data[i].code != NULL; i++)
    {
        if (strncmp (locale, locale_data[i].code, strlen (locale)) == 0)
        {
            *kbd = g_strdup (locale_data[i].kbd);
            *tz = g_strdup (locale_data[i].tz);
            found = TRUE;
            break;
        }
    }
    /* we shouldn't get here */
    if (!found)
    {
        *kbd = NULL;
        *tz = NULL;
    }
}

gboolean
get_service_is_blacklisted (const gchar *service)
{
    gboolean retval = FALSE;
    const gchar *file = "/cdrom/boot.cfg";
    gchar *str = NULL;
    XfceRc *rc;

    rc = xfce_rc_simple_open (file, TRUE);
    if (!rc)
        return FALSE;

    str = g_shell_unquote (xfce_rc_read_entry (rc, "SERVICES_BLACKLIST", " "), NULL);
    xfce_rc_close (rc);

    if (!str)
        return FALSE;

    gchar **blacklist = NULL;
    int size = 0, i;
    blacklist = g_strsplit (str, " ", 0);
    size = g_strv_length (blacklist);

    for (i=0;i<size;i++)
        if (!strcmp (blacklist[i], service))
        {
            retval = TRUE;
            break;
        }

    g_strfreev (blacklist);
    g_free (str);

    return retval;
}

gboolean
get_autostart_infos (gchar **command, gchar **icon)
{
    XfceRc *rc;
    XfceDesktopEntry *de;
    gchar *filename = NULL;
    gchar *home = NULL;
    const gchar *default_app;
    const gchar *default_command;

    home = get_home ();
    filename = g_strdup_printf ("%s/.config/appstarterrc", home);
    g_free (home);

    rc = xfce_rc_simple_open (filename, TRUE);
    g_free (filename);

    if (! rc)
    {
        rc = xfce_rc_simple_open ("/etc/xdg/appstarterrc", TRUE);
    }

    if (!rc)
        return FALSE;

    xfce_rc_set_group (rc, "default");
    default_app = xfce_rc_read_entry (rc, "selected", "none");
    default_command = xfce_rc_read_entry (rc, "command", "true");
    xfce_rc_close (rc);

    /* command as true and app as firefox means default config, we set the
       selection to none */
    if ((g_strcmp0 (default_app, "none") == 0)
        || ((g_strcmp0 (default_app, "firefox") == 0)
            && (g_strcmp0 (default_command, "true") == 0)))
    {
        g_free (*command);
        *command = g_strdup ("None");
        g_free (*icon);
        /* cf appstarter code */
        *icon = g_strdup ("/usr/share/icons/Rodent/48x48/stock/stock_cancel.png");

        return TRUE;
    }

    filename = g_strdup_printf (
        "/usr/share/appstarter/plugins/%s-plugin.desktop", default_app);
    de = xfce_desktop_entry_new (filename, dentry_keys,
                                    G_N_ELEMENTS (dentry_keys));
    g_free (filename);

    if (!de)
        return FALSE;

    xfce_desktop_entry_get_string (de, "GenericName", TRUE, command);
    xfce_desktop_entry_get_string (de, "Icon", TRUE, icon);

    g_object_unref (de);

    return TRUE;
}

void
restore_dialog_run ()
{
    gchar *cmd;

    if (xfce_confirm (_("Would you really like to restore your system now?\n"
                        "\n"
                        "The system will reboot if you continue.\n"
                        "\n"
                        "A menu will offer you to select the restoration mode."),
                      GTK_STOCK_YES, NULL))
    {
        cmd = g_strdup_printf ("sh %s/restore.sh restore", LIBEXECDIR);
        if (system (cmd) == 0)
            system (REBOOTCMD);
    }
}

#define SCHEDFILE "/cdrom/etc/linutopd/scheduler/jobs"
gboolean
reboot_scheduled (guint *hours, guint *minutes)
{
    gchar *contents = NULL;
    gchar **lines = NULL;
    gchar **linep;
    gchar **words;
    gboolean ret = FALSE;

    if (! g_file_get_contents (SCHEDFILE, &contents, NULL, NULL))
        return FALSE;

    lines = g_strsplit (contents, "\n", 0);
    g_free (contents);
    for (linep = lines; *linep; linep++)
    {
        if (**linep == '#')
            continue;

        words = g_strsplit (*linep, " ", 0);
        if (3 != g_strv_length (words))
        {
            g_strfreev (words);
            continue;
        }

        if (g_strcmp0 (words[2], "reboot") == 0)
        {
            *hours = atoi (words[1]);
            *minutes = atoi (words[0]);

            ret = TRUE;
            g_strfreev (words);
            break;
        }

        g_strfreev (words);
    }
    g_strfreev (lines);

    return ret;
}

