/**
 * Copyright (C) 2010 Gauvain Pocentek <gpocentek@linutop.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif

#ifdef HAVE_SYS_TYPES_H
#include <sys/types.h>
#endif

#ifdef HAVE_SIGNAL_H
#include <signal.h>
#endif

#include <libxfce4util/libxfce4util.h>

#include "lgfind.h"

enum
{
    WORD_CONTAINS = 0,
    WORD_STARTS_WITH,
    WORD_ENDS_WITH,
    WORD_EXACTLY,
    WORD_MAX
};

static gchar *name_types[] =
{
    N_("containing"),
    N_("starting with"),
    N_("ending with"),
    N_("named"),
};

gchar *init_directory = NULL;

/*
 * Callbacks
 */
static void
on_window_delete (GtkWidget *window, GdkEvent *event, gpointer data)
{
    LGFind *app = (LGFind *) data;

    if (app->pid)
        kill (app->pid, SIGTERM);
    if (app->channel)
    {
        g_io_channel_shutdown (app->channel, TRUE, NULL);
        g_io_channel_unref (app->channel);
        app->channel = NULL;
    }

    gtk_main_quit ();
}

static void
on_row_activated (GtkTreeView *treeview, GtkTreePath *path,
                  GtkTreeViewColumn *column, gpointer data)
{
    GtkTreeIter iter;
    GtkTreeModel *model;
    gchar *str = NULL;
    gchar *cmd = NULL;

    model = gtk_tree_view_get_model (treeview);
    if (!gtk_tree_model_get_iter (model, &iter, path))
        return;

    gtk_tree_model_get (model, &iter, 0, &str, -1);
    cmd = g_strdup_printf ("xdg-open %s", str);
    g_spawn_command_line_async (cmd, NULL);
    g_free (str);
    g_free (cmd);
}

static gboolean
on_find_result (GIOChannel *channel, GIOCondition condition, gpointer data)
{
    GtkTreeModel *model;
    GtkTreeIter iter;
    gchar *str = NULL;
    gsize eol;
    LGFind *app = (LGFind *) data;

    if (condition == G_IO_HUP)
    {
        g_io_channel_shutdown (app->channel, TRUE, NULL);
        g_io_channel_unref (app->channel);
        app->channel = NULL;
        gtk_widget_set_sensitive (app->find_btn, TRUE);
        gtk_widget_set_sensitive (app->cancel_btn, FALSE);
        close (app->out);
        return FALSE;
    }

    g_io_channel_read_line (channel, &str, NULL, &eol, NULL);
    str[eol] = '\0';

    model = gtk_tree_view_get_model (GTK_TREE_VIEW (app->result_treeview));
    gtk_list_store_append (GTK_LIST_STORE (model), &iter);
    gtk_list_store_set (GTK_LIST_STORE (model), &iter, 0, str, -1);
    g_free (str);

    return TRUE;
}

static void
run_find_cmd (LGFind *app)
{
    gchar **argv = NULL;
    gint argc;
    gint out;
    GPid pid;
    GError *error = NULL;

    if (!g_shell_parse_argv (app->cmd, &argc, &argv, NULL))
        return;

    if (!g_spawn_async_with_pipes (NULL, argv, NULL,
                                   G_SPAWN_DO_NOT_REAP_CHILD | G_SPAWN_SEARCH_PATH,
                                   NULL, app, &pid, NULL, &out, NULL, &error))
    {
        printf ("%s\n", error->message);
        g_error_free (error);
        g_strfreev (argv);
        return;
    }

    gtk_widget_set_sensitive (app->find_btn, FALSE);
    gtk_widget_set_sensitive (app->cancel_btn, TRUE);
    gtk_list_store_clear (GTK_LIST_STORE (gtk_tree_view_get_model (GTK_TREE_VIEW (app->result_treeview))));

    app->out = out;
    app->pid = pid;
    app->channel = g_io_channel_unix_new (app->out);
    g_io_channel_set_close_on_unref (app->channel, TRUE);
    g_io_add_watch (app->channel, G_IO_IN | G_IO_HUP, (GIOFunc) on_find_result, app);

    g_strfreev (argv);
}

static void
on_cancel_btn_clicked (GtkWidget *btn, gpointer data)
{
    LGFind *app = (LGFind *) data;

    if (app->pid)
        kill (app->pid, SIGTERM);
    if (app->channel)
    {
        g_io_channel_shutdown (app->channel, TRUE, NULL);
        g_io_channel_unref (app->channel);
        app->channel = NULL;
        gtk_widget_set_sensitive (app->find_btn, TRUE);
        gtk_widget_set_sensitive (app->cancel_btn, FALSE);
        close (app->out);
    }
}

static void
on_find_btn_clicked (GtkWidget *btn, gpointer data)
{
    gchar *tmp = NULL;
    gchar *cmd = NULL;

    /* MUST NOT be freed */
    const gchar *dir = NULL;

    gchar *name = NULL;

    GtkTreeIter iter;
    GtkTreeModel *model;
    int word_type;

    LGFind *app = (LGFind *) data;

    dir = gtk_entry_get_text (GTK_ENTRY (app->dir_entry));
    name = g_strdup (gtk_entry_get_text (GTK_ENTRY (app->name_entry)));

    model = gtk_combo_box_get_model (GTK_COMBO_BOX (app->name_combo));
    gtk_combo_box_get_active_iter (GTK_COMBO_BOX (app->name_combo), &iter);
    gtk_tree_model_get (model, &iter, 0, &word_type, -1);

    if (! name || (g_strcmp0 (name, "") == 0))
        goto out;

    switch (word_type)
    {
        case WORD_STARTS_WITH:
            tmp = g_strdup_printf ("-name \"%s*\"", name);
            break;
        case WORD_ENDS_WITH:
            tmp = g_strdup_printf ("-name \"*%s\"", name);
            break;
        case WORD_EXACTLY:
            tmp = g_strdup_printf ("-name \"%s\"", name);
            break;
        case WORD_CONTAINS:
            tmp = g_strdup_printf ("-name \"*%s*\"", name);
            break;
    }

    if (! dir || (g_strcmp0 (dir, "") == 0))
    {
        dir = g_get_home_dir ();
    }

    cmd = g_strdup_printf ("find %s %s", dir, tmp);
    g_free (app->cmd);
    app->cmd = cmd;
    if (app->pid)
    {
        kill (app->pid, SIGTERM);
        app->pid = 0;
    }
    if (app->channel)
    {
        g_io_channel_shutdown (app->channel, TRUE, NULL);
        g_io_channel_unref (app->channel);
        app->channel = NULL;
    }

    run_find_cmd (app);

out:
    g_free (name);
    name = NULL;
    g_free (tmp);
    tmp = NULL;
}

static void
populate (LGFind *app)
{
    GtkListStore *name_combo_store;
    GtkListStore *result_treeview_store;

    GtkTreeViewColumn *column;
    GtkCellRenderer *renderer;
    GtkTreeIter iter;
    int i;

    /* name combo */
    renderer = gtk_cell_renderer_text_new ();
    gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (app->name_combo),
                                renderer, FALSE);
    gtk_cell_layout_add_attribute (GTK_CELL_LAYOUT (app->name_combo),
                                renderer, "text", 1);
    name_combo_store = gtk_list_store_new (2, G_TYPE_INT, G_TYPE_STRING);
    gtk_combo_box_set_model (GTK_COMBO_BOX (app->name_combo),
                             GTK_TREE_MODEL (name_combo_store));
    for (i = 0; i < WORD_MAX; i++)
    {
        gtk_list_store_append (name_combo_store, &iter);
        gtk_list_store_set (name_combo_store, &iter,
                            0, i, 1, _(name_types[i]), -1);
    }
    gtk_combo_box_set_active (GTK_COMBO_BOX (app->name_combo), 0);

    result_treeview_store = gtk_list_store_new (1, G_TYPE_STRING);
    gtk_tree_view_set_model (GTK_TREE_VIEW (app->result_treeview),
                             GTK_TREE_MODEL (result_treeview_store));
    renderer = gtk_cell_renderer_text_new ();
    column = gtk_tree_view_column_new_with_attributes ("", renderer, "text", 0, NULL);
    gtk_tree_view_insert_column (GTK_TREE_VIEW (app->result_treeview), column, 0);
    gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (app->result_treeview), FALSE);
}

int
main (gint argc, gchar **argv)
{
    LGFind *app;

    GtkWidget *window;
    GtkWidget *hbox, *vbox;
    GtkWidget *scw;

    static GOptionEntry entries[] =
    {
        { "directory", 'd', 0, G_OPTION_ARG_STRING, &init_directory,
          "Directory in which the search will happen", "DIR" },
        { NULL }
    };

    xfce_textdomain (GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR, "UTF-8");

    gtk_init_with_args (&argc, &argv, "", entries, NULL, NULL);

    app = g_new (LGFind, 1);
    app->cmd = NULL;
    app->pid = 0;
    app->channel = NULL;

    app->name_combo = gtk_combo_box_new ();
    app->name_entry = gtk_entry_new ();
    app->dir_entry = gtk_entry_new ();
    app->result_treeview = gtk_tree_view_new ();
    app->find_btn = gtk_button_new_from_stock (GTK_STOCK_FIND);
    app->cancel_btn = gtk_button_new_from_stock (GTK_STOCK_CANCEL);
    gtk_widget_set_sensitive (app->cancel_btn, FALSE);

    gtk_window_set_default_icon_name ("find");
    window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    vbox = gtk_vbox_new (FALSE, 0);
    gtk_container_add (GTK_CONTAINER (window), vbox);

    hbox = gtk_hbox_new (FALSE, 0);
    gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);

    gtk_box_pack_start (GTK_BOX (hbox),
                        gtk_label_new (_("Search files")),
                        FALSE, FALSE, 2);
    gtk_box_pack_start (GTK_BOX (hbox), app->name_combo, FALSE, FALSE, 2);
    gtk_box_pack_start (GTK_BOX (hbox), app->name_entry, TRUE, TRUE, 2);
    gtk_box_pack_start (GTK_BOX (hbox),
                        gtk_label_new (_("in folder")),
                        FALSE, FALSE, 2);
    gtk_box_pack_start (GTK_BOX (hbox), app->dir_entry, TRUE, TRUE, 2);
    gtk_box_pack_end (GTK_BOX (hbox), app->cancel_btn, FALSE, FALSE, 2);
    gtk_box_pack_end (GTK_BOX (hbox), app->find_btn, FALSE, FALSE, 2);

    scw = gtk_scrolled_window_new (NULL, NULL);
    gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scw),
                                    GTK_POLICY_AUTOMATIC,
                                    GTK_POLICY_AUTOMATIC);
    gtk_container_add (GTK_CONTAINER (scw), app->result_treeview);
    gtk_box_pack_start (GTK_BOX (vbox), scw, TRUE, TRUE, 2);

    populate (app);
    if (init_directory)
    {
        gtk_entry_set_text (GTK_ENTRY (app->dir_entry), init_directory);
        g_free (init_directory);
    }

    gtk_widget_show_all (window);

    g_signal_connect (G_OBJECT (window), "delete-event",
                      G_CALLBACK (on_window_delete), app);
    g_signal_connect (G_OBJECT (app->cancel_btn), "clicked",
                      G_CALLBACK (on_cancel_btn_clicked), app);
    g_signal_connect (G_OBJECT (app->find_btn), "clicked",
                      G_CALLBACK (on_find_btn_clicked), app);
    g_signal_connect (G_OBJECT (app->result_treeview), "row-activated",
                      G_CALLBACK (on_row_activated), app);

    gtk_main ();

    g_free (app);

    return 0;
}
